@@ -210,11 +210,11 @@ void *thr_reader(void *_count)
 		debug_yield_read();
 		if (local_ptr)
 			assert(local_ptr->a == 8);
-		if (unlikely(rduration))
+		if (caa_unlikely(rduration))
 			loop_sleep(rduration);
 		rcu_read_unlock();
 		nr_reads++;
-		if (unlikely(!test_duration_read()))
+		if (caa_unlikely(!test_duration_read()))
 			break;
 	}
 
@@ -261,7 +261,7 @@ void *thr_writer(void *data)
 		new = malloc(sizeof(*new));
 		new->a = 8;
 		old = rcu_xchg_pointer(&test_rcu_pointer, new);
-		if (unlikely(wduration))
+		if (caa_unlikely(wduration))
 			loop_sleep(wduration);
 		defer_rcu(free, old);
 		defer_rcu(test_cb1, old);
@@ -272,9 +272,9 @@ void *thr_writer(void *data)
 		defer_rcu(test_cb2, (void *)-4L);
 		defer_rcu(test_cb2, (void *)-2L);
 		nr_writes++;
-		if (unlikely(!test_duration_write()))
+		if (caa_unlikely(!test_duration_write()))
 			break;
-		if (unlikely(wdelay))
+		if (caa_unlikely(wdelay))
 			loop_sleep(wdelay);
 	}
 
