@@ -20,8 +20,8 @@
 
 #include <stddef.h>	/* for offsetof */
 
-#define likely(x)	__builtin_expect(!!(x), 1)
-#define unlikely(x)	__builtin_expect(!!(x), 0)
+#define caa_likely(x)	__builtin_expect(!!(x), 1)
+#define caa_unlikely(x)	__builtin_expect(!!(x), 0)
 
 #define	cmm_barrier()	asm volatile("" : : : "memory")
 
@@ -39,12 +39,12 @@
  */
 #define CMM_ACCESS_ONCE(x)	(*(volatile typeof(x) *)&(x))
 
-#ifndef max
-#define max(a,b) ((a)>(b)?(a):(b))
+#ifndef caa_max
+#define caa_max(a,b) ((a)>(b)?(a):(b))
 #endif
 
-#ifndef min
-#define min(a,b) ((a)<(b)?(a):(b))
+#ifndef caa_min
+#define caa_min(a,b) ((a)<(b)?(a):(b))
 #endif
 
 #if defined(__SIZEOF_LONG__)
@@ -55,10 +55,34 @@
 #define CAA_BITS_PER_LONG	32
 #endif
 
-#define caa_container_of(ptr, type, member)					\
+/*
+ * caa_container_of - Get the address of an object containing a field.
+ *
+ * @ptr: pointer to the field.
+ * @type: type of the object.
+ * @member: name of the field within the object.
+ */
+#define caa_container_of(ptr, type, member)				\
 	({								\
-		const typeof(((type *)NULL)->member) * __ptr = (ptr);	\
+		const typeof(((type *) NULL)->member) * __ptr = (ptr);	\
 		(type *)((char *)__ptr - offsetof(type, member));	\
 	})
 
+#define CAA_BUILD_BUG_ON_ZERO(cond) (sizeof(struct { int:-!!(cond); }))
+#define CAA_BUILD_BUG_ON(cond) ((void)BUILD_BUG_ON_ZERO(cond))
+
+/*
+ * __rcu is an annotation that documents RCU pointer accesses that need
+ * to be protected by a read-side critical section. Eventually, a static
+ * checker will be able to use this annotation to detect incorrect RCU
+ * usage.
+ */
+#define __rcu
+
+#ifdef __cplusplus
+#define URCU_FORCE_CAST(type, arg)	(reinterpret_cast<type>(arg))
+#else
+#define URCU_FORCE_CAST(type, arg)	((type) (arg))
+#endif
+
 #endif /* _URCU_COMPILER_H */
